"""
Functionality for View and NetCDF setting menus.

Creates the dropdown lists for the 'View' and 'NetCDF' setting menus. Manages the execution
of setting changes when they occur. This module only defines the dynamic aspect of the menus.
Static aspects are in the nccut.kv file.
"""

from kivy.uix.button import Button
from kivy.uix.dropdown import DropDown
from kivy.app import App


class ViewDropDown(DropDown):
    """
    Dynamic elements of the 'View' settings menu.

    Creates the line color dropdown menu and manages the execution of all setting options.

    Attributes:
        home: Reference to root :class:`nccut.homescreen.HomeScreen` instance
        l_color_drop: Line color selection kivy.uix.dropdown.Dropdown object
    """

    def __init__(self, **kwargs):
        """
        Connects to root HomeScreen instance and creates line color dropdown.
        """
        super(ViewDropDown, self).__init__(**kwargs)
        self.home = App.get_running_app().root.get_screen("HomeScreen")

        col_list = ["Blue", "Orange", "Green"]
        self.l_color_drop = DropDown()
        for i in col_list:
            btn = Button(text=i, size_hint_y=None, height=30)
            btn.bind(on_release=lambda btn: self.pass_setting("l_color", btn.text))  # Setting name: 'l_color'
            btn.bind(on_press=self.l_color_drop.dismiss)
            self.l_color_drop.add_widget(btn)

    def pass_setting(self, setting, value):
        """
        Passes setting changes to the viewer.

        Args:
            setting (str): Name of setting being changed.
            value: New setting value of appropriate data type for setting
        """
        if self.home.file_on:
            self.home.display.update_settings(setting, value)

    def rotate(self):
        """
        Call for a 45 degree rotation of current display by 45 degrees
        """
        if self.home.file_on:
            self.home.display.rotate()

    def flip_v(self):
        """
        Call for a vertical flip of view of current display
        """
        if self.home.file_on:
            self.home.display.flip_vertically()

    def flip_h(self):
        """
        Call for a horizontal flip of view of current display
        """
        if self.home.file_on:
            self.home.display.flip_horizontally()


class NetCDFDropDown(DropDown):
    """
    Dynamic elements of the NetCDF setting menu

    Creates all dropdown lists and executes all setting changes. Other settings are
    defined statically in nccut.kv file.

    Attributes:
        home: Reference to root :class:`nccut.homescreen.HomeScreen` instance
        cmap_dropdown: Color map selection kivy.uix.dropdown.Dropdown object
        var_dropdown: Variable selection kivy.uix.dropdown.Dropdown object
        depth_dropdown: Z dimension value selection kivy.uix.dropdown.Dropdown object
    """
    def __init__(self, **kwargs):
        """
        Connects to root HomeScreen instance and defines dropdown menus.

        Colormaps are defined in :class:`nccut.filedisplay.FileDisplay` class. Variables and z dimension values come
        from the currently loaded NetCDF file.
        """
        super(NetCDFDropDown, self).__init__(**kwargs)
        self.home = App.get_running_app().root.get_screen("HomeScreen")
        if self.home.file_on:
            f_type = self.home.display.f_type
        else:
            f_type = None

        self.cmap_dropdown = DropDown()
        if self.home.file_on and f_type == "netcdf":
            for i in list(self.home.display.cmaps.keys()):
                btn = Button(text=i, size_hint_y=None, height=30)
                btn.bind(on_release=lambda btn: self.pass_setting("colormap", btn.text))  # Setting name: 'colormap'
                btn.bind(on_press=self.cmap_dropdown.dismiss)
                self.cmap_dropdown.add_widget(btn)

        self.var_dropdown = DropDown()
        if self.home.file_on and f_type == "netcdf":
            for i in list(self.home.display.config["netcdf"]["file"].keys()):
                btn = Button(text=i, size_hint_y=None, height=30)
                btn.bind(on_press=lambda btn: self.pass_setting("variable", btn.text))  # Setting name: 'variable'
                btn.bind(on_release=self.var_dropdown.dismiss)
                self.var_dropdown.add_widget(btn)

        self.depth_dropdown = DropDown()
        if self.home.file_on and f_type == "netcdf" and self.home.display.config['netcdf']['z'] != "N/A":
            for i in list(self.home.display.config["netcdf"]['file'][self.home.display.config["netcdf"]['z']].data):
                btn = Button(text=str(i), size_hint_y=None, height=30)
                btn.bind(on_press=lambda btn: self.pass_setting("depth", btn.text))  # Setting name: 'depth'
                btn.bind(on_release=self.depth_dropdown.dismiss)
                self.depth_dropdown.add_widget(btn)

    def pass_setting(self, setting, value):
        """
        Pass setting changes to display.

        Args:
            setting (str): Name of setting being changed.
            value: New setting value of appropriate data type for setting
        """
        if self.home.file_on:
            self.home.display.update_settings(setting, value)



"""
Widget for Transect tool

Manages having multiple SingleTransects on screen at once.
"""
import kivy.uix as ui
from nccut.plotpopup import PlotPopup
from nccut.singletransect import SingleTransect
import nccut.functions as func
from kivy.core.window import Window


class MultiTransect(ui.widget.Widget):
    """
    Creates, stores, and manages multiple SingleTransects

    Created when Transect button is selected. From there on this object manages the creation, modification,
    and data packaging of transects made by user. Also used by marker to manage SingleTransects made
    orthogonally to user marked line.

    Attributes:
        lines (list): List of all :class:`nccut.singletransect.SingleTransect` made.
        clicks (int): Number of clicks made by user. Cycles between 0 and 2 depending on current stage of
            transect drawing.
        home: Reference to root :class:`nccut.homescreen.HomeScreen` instance
        p_btn: RoundedButton, Plot button which opens :class:`nccut.plotpopup.PlotPopup`
        dragging (bool): Whether in dragging mode
        plotting: :class:`nccut.plotpopup.PlotPopup`, reference to plotting menu when opened
    """
    def __init__(self, home, **kwargs):
        """
        Initialize object and create Plot button.

        Args:
            home: Reference to root :class:`nccut.homescreen.HomeScreen` instance
        """
        super(MultiTransect, self).__init__(**kwargs)
        self.lines = []
        self.clicks = 0
        self.home = home
        self.p_btn = func.RoundedButton(text="Plot", size_hint=(1, 0.1), font_size=self.home.font)
        self.p_btn.bind(on_press=lambda x: self.popup())
        self.dragging = False
        self.plotting = None

    def font_adapt(self, font):
        """
        Updates font size of Plot button.

        Args:
            font (float): New font size
        """
        self.p_btn.font_size = font

    def update_l_col(self, value):
        """
        Asks each transect to update their line color

        Args:
            value (str): New color value: 'Blue', 'Green' or 'Orange'
        """
        for t in self.children:
            t.update_l_col(value)

    def update_c_size(self, value):
        """
       Asks each transect to update their circle size

       Args:
           value (float): New circle size
       """
        for t in self.children:
            t.update_c_size(value)

    def del_line(self):
        """
        Delete all clicked points in most recent transect
        """
        for i in range(self.clicks):
            self.del_point()

    def del_point(self):
        """
        Delete most recent clicked point
        """
        if len(self.lines) == 0:  # If no transects, do nothing
            return
        elif self.children[0].circles == 1:
            # Case where point is beginning of a transect
            Window.unbind(mouse_pos=self.children[0].draw_line)
            self.remove_widget(self.children[0])
            self.lines = self.lines[:-1]
            # Add plot button to sidebar if not the last transect
            if len(self.lines) != 0:
                self.home.display.current.insert(0, self.p_btn)
        elif self.children[0].circles == 2:
            # Case where point is end of a transect
            self.children[0].del_point()
            if self.p_btn in self.home.display.current:
                # Remove plot button from sidebar
                self.home.display.current.remove(self.p_btn)
        if self.clicks == 2:
            self.clicks = 0
        self.clicks += 1

    def change_dragging(self, val):
        """
        Update whether in dragging mode

        Args:
            val (bool): Whether in dragging mode or not
        """
        self.dragging = val

    def popup(self):
        """
        Gathers coordinates from all :class:`nccut.singletransect.SingleTransect` into a dictionary and calls for
        popup
        """
        data = {}
        count = 1
        for i in self.lines:
            data["Cut " + str(count)] = i.line.points
            count += 1
        # Open plotting popup
        self.plotting = PlotPopup({"Multi": data}, self.home, self.home.display.config)

    def on_touch_down(self, touch):
        """
        Creating transects and managing Plot button based on which of 3 click stages user is in.

        Args:
            touch: MouseMotionEvent, see kivy docs for details
        """
        if not self.dragging:
            if self.home.ids.view.collide_point(*self.home.ids.view.to_widget(*self.to_window(*touch.pos))):
                if self.clicks == 2:
                    # Clean up Plot button from previous cycle and reset
                    self.clicks = 0
                    self.home.ids.sidebar.remove_widget(self.p_btn)
                if self.clicks == 0:
                    # Begins a new transect
                    x = SingleTransect(home=self.home)

                    self.add_widget(x)
                    self.lines.append(x)
                # If clicked same point as before, do nothing
                elif [touch.x, touch.y] == self.lines[-1].line.points:
                    return
                # SingleTransect manages the line and dots graphics
                self.lines[-1].on_touch_down(touch)
                if self.clicks == 1:
                    # Finishes a transect, displays download button
                    if self.p_btn not in self.home.ids.sidebar.children:
                        self.home.ids.sidebar.add_widget(self.p_btn, 1)
                self.clicks += 1



"""
Functionality for the main screen as well as root of widget tree.

This module manages the functionality of the home screen elements. It also serves as
the root node for the app and passes necessary commands to their appropriate recipient
further down the tree.

"""
import kivy
import numpy as np
from kivy.app import App
from kivy.uix.screenmanager import Screen
import re
import os
import platform
import nccut.functions as func
from pathlib import Path
from nccut.filedisplay import FileDisplay
from nccut.netcdfconfig import NetCDFConfig


class HomeScreen(Screen):
    """
    Manages the dynamic functionality of the main screen UI elements. Creates and calls upon other modules
    to execute GUI operations.

    Attributes:
        file_on (bool): Whether there is a file currently loaded in viewer
        loaded (bool): Whether or not the window has fully loaded
        win_load_size: Height and width of fully loaded window on Windows OS
        rel_path: pathlib.Path object to use as output directory
        font (float): Current font size for all buttons
        display: FileDisplay object (draggable image) created when a file is loaded
        nc_popup: Reference to NetCDF configuration popup
        file: File if one was given on start up from command line, otherwise None
    """
    def __init__(self, file=None, **kwargs):
        """
        Initialize main screen with default settings.

        Creates editing buttons, most UI elements defined in nccut.kv

        Args:
            file: (Optional) File if one was given on start up from command line, otherwise None
        """
        super(HomeScreen, self).__init__(**kwargs)
        self.file_on = False
        self.loaded = False
        self.win_load_size = np.array([899.3000000000001, 525.21])
        self.mac_load_size = np.array([1540.0, 902.0000000000001])
        self.rel_path = Path(os.getcwd())
        self.font = self.ids.transect.font_size
        self.display = None
        self.nc_popup = None
        self.file = file

    def initial_load(self):
        """
        If a file was given on start up, wait until app is fully loaded and then load given file.
        """
        if platform.system() == "Darwin":
            load_size = self.mac_load_size
        else:
            load_size = self.win_load_size
        if not self.loaded and np.array_equal(np.array(self.ids.view.size), load_size):
            if self.file:
                self.ids.file_in.text = str(self.file)
                self.go_btn()
                if str(self.file)[-3:] == ".nc":
                    self.nc_popup.go.dispatch("on_press")
            self.loaded = True

    def font_adapt(self):
        """
        Updates font size throughout widget tree to the size automatically determined for static UI elements.

        When the window is resized Kivy automatically updates the font size of UI elements defined in nccut.kv
        but not those defined in scripts. Thus this method updates font size for such elements to be the same as
        the static elements.
        """
        font = self.ids.transect.font_size

        self.font = font
        self.ids.view_btn.font_size = font
        self.ids.netcdf_btn.font_size = font
        if self.file_on:
            self.display.font_adapt(font)

    def transect_btn(self, t_type):
        """
        Calls for the creation and deletion of tools.
        Args:
            t_type (str): Tool type: 'transect' or 'transect_marker'
        """
        if self.file_on:
            self.font_adapt()
            self.display.manage_tool(t_type)

    def go_btn(self):
        """
        If file name is valid and exists, load image or NetCDF File.

        If a file and any transect tools are already loaded, first clears viewer.
        """
        if self.file_on:
            self.display.reset_sidebar()
            self.clean_file()

        file = self.ids.file_in.text

        # Limit file names to alphanumeric characters and _-./
        if file == "" or len(re.findall(r'[^A-Za-z0-9_:\\.\-/]', file)) > 0:
            func.alert("Invalid File Name", self)
            self.clean_file()
        else:
            if len(file) >= 1:
                try:
                    open(file)
                    if file[-3:] == ".nc":
                        # Creates selection popup for nc file data sets
                        self.nc_popup = NetCDFConfig(file, self)

                    elif file[-5:] == ".jpeg" or file[-4:] == ".png" or file[-4:] == ".jpg":
                        # Creates interactive image from .jpg/.png/.jpeg files
                        self.display = FileDisplay(home=self, f_config={"image": str(file)})
                        self.ids.view.add_widget(self.display)
                        self.file_on = True
                    else:
                        func.alert("Unsupported File Type", self)
                        self.clean_file()

                except FileNotFoundError:
                    func.alert("File Not Found", self)
                    self.clean_file()

    def load_netcdf(self, config):
        """
        Load NetCDF file

        Args:
            config (dict): Dictionary of verified NetCDF file configuration settings. Check FileDisplay for more details
        """
        self.display = FileDisplay(home=self, f_config={"netcdf": config})
        self.ids.view.add_widget(self.display)
        self.file_on = True

    def clean_file(self):
        """
        Resets file related attributes.
        """
        kivy.core.window.Window.set_system_cursor("arrow")
        if len(self.ids.colorbar.children) != 0:
            self.ids.colorbar.remove_widget(self.ids.colorbar.children[0])
        if self.file_on:
            self.display.parent.remove_widget(self.display)
        self.file_on = False

    def canvas_remove(self, item, *largs):
        """
        Allows external sources to clear canvas.

        Args:
            item: Canvas item to be removed
            *largs: Any other args that are passed

        """
        self.canvas.remove(item)

    def quit_btn(self):
        """
        Quit application
        """
        App.get_running_app().stop()



"""
Miscellaneous helper functions to support other widgets.

Provides reference to custom UI elements defined in nccut.kv file so they can be used in python scripts.
Contains error banner functionality, file management function, and transect taking function.

"""

import kivy
from kivy.graphics import Color, Rectangle
from kivy.uix.button import Button
from kivy.uix.label import Label
from functools import partial
from scipy.interpolate import RegularGridInterpolator
import numpy as np
import math
import re
from pathlib import Path


class RoundedButton(Button):
    """
    Code for this is in nccut.kv. Referenced here so it can be used in scripts.
    """
    pass


class BackgroundLabel(Label):
    """
    Code for this is in nccut.kv. Referenced here so it can be used in scripts.
    """
    pass


def text_wrap(*args):
    """
    Updates a widgets text box so that it is always within the bounds of the widget.

    Args:
        args: List where first item is a Button or Label and the second item is a tuple of that widgets current size.
    """
    args[0].text_size = (args[1][0] - 8, args[1][1] - 8)


def remove_alert(alert, home, *largs):
    """
    Remove alert banner.

    Args:
        alert: Alert kivy.uix.label.Label reference.
        home: Active :class:`nccut.homescreen.HomeScreen` instance.
        *largs: Unused args from kivy Clock class.
    """
    home.remove_widget(alert)


def alert(text, home):
    """
    Creates alert banner.

    Creates alert banner with given text and adds it to top left corner of the home screen.
    Schedules it to be removed after 2 seconds.

    Args:
        text: String of alert banner message
        home: Active :class:`nccut.homescreen.HomeScreen` reference
    """

    screen = kivy.core.window.Window.size
    with home.canvas:
        Color(0.2, 0.2, 0.2)
        box = Rectangle(pos=(10, kivy.core.window.Window.size[1] - 60), size=(300, 50))
    aler = Label(text=text, size=home.size, pos=(-(screen[0] / 2) + 160, screen[1] / 2 - 35))
    home.add_widget(aler)
    kivy.clock.Clock.schedule_once(partial(remove_alert, aler, home), 2)
    kivy.clock.Clock.schedule_once(partial(home.canvas_remove, box), 2)


def check_file(path, fname, extension):
    """
    Checks if a filename is valid and prevents overwriting.

    Checks a file name doesn't have any problematic characters. If file name is a file path
    ensures that the directories exists. If a file name is the same as one that already
    exists it adds a (#) to avoid overwriting existing file.

    Args:
        path: pathlib.Path object of current output directory
        fname (str): User proposed file name
        extension (str): File extension for file type being created

    Returns:
        If all checks are passed the file name is returned, possibly with added (#). If checks aren't passed
        returns False.
    """
    if fname.find(".") >= 1:
        fname = fname[:fname.find(".")]
    if fname == "" or len(re.findall(r'[^A-Za-z0-9_\-/:]', fname)) > 0:
        return False
    if "/" in fname:
        if not Path.exists(path / fname[:fname.rfind("/") + 1]):
            return False

    exist = True
    fcount = 0
    while exist:
        if Path.exists(path / (fname + extension)):
            fcount += 1
            if fcount == 1:
                fname = fname + "(1)"
            else:
                fname = fname[:fname.find("(") + 1] + str(fcount) + ")"
        else:
            exist = False

    return fname


def sel_data(config):
    """
    Selects data from NetCDF file according to dimension and variable selections.

    Args:
        config: Dictionary of key details about NetCDF file as outlined in
            :meth:`nccut.netcdfconfig.NetCDFConfig.check_inputs`

    Returns:
        2D array of data from NetCDF file.
    """
    # IMPORTANT: Numpy indexes (row, column) with (0, 0) at top left corner
    if config['z'] == 'N/A':
        # 2D NetCDF data
        ds = config['file'][config['var']].rename({config['y']: 'y', config['x']: 'x'})
        ds = ds.transpose('y', 'x')
        data = ds.sel(y=ds['y'], x=ds['x'])
    else:
        # 3D NetCDF data
        ds = config['file'][config['var']].rename({config['y']: 'y', config['x']: 'x', config['z']: 'z'})
        ds = ds.transpose('y', 'x', 'z')
        ds['z'] = ds['z'].astype(str)
        data = ds.sel(y=ds['y'], x=ds['x'], z=config['z_val'])
    return np.flip(data.data, 0)


def ip_get_points(points, curr, nc):
    """
    Creates a data frame containing x, y, and value of points on transect line

    From end points finds points of line connecting them. Interpolates higher resolution version of
    relevant section of data set, and then grabs data values at each line point.

    Args:
        points: 4 element 1D array with coordinates of the two transect end points: [X1, Y1, X2, Y2]
        curr: 2D indexable array of current dataset loaded in viewer
        nc (bool): Whether data is from a NetCDF file
    Returns:
        Dictionary with three keys:
            'x': 1D array of x-coordinates of transect points
            'y': 1D array of y-coordinates of transect points
            'Cut': 1D array of data values at each point along line connecting end points
    """

    # If angle of line is > 45 degrees will swap x and y to increase accuracy
    data = []
    xyswap = False

    img = curr
    line = points

    # Always read from left point to right
    if line[0] > line[2]:
        line = [line[2], line[3], line[0], line[1]]

    # Get interpolation object

    # Get x values
    ix = np.arange(int(line[0] - 3), int(line[2] + 4))
    # Get y values increasing in value w/o changing og object
    if line[1] > line[3]:
        iy = np.arange(int(line[3] - 3), int(line[1] + 4))
    else:
        iy = np.arange(int(line[1] - 3), int(line[3] + 4))
    # Get line slope
    if line[2] - line[0] == 0:
        m = (line[3] - line[1]) / .001
    else:
        m = (line[3] - line[1]) / (line[2] - line[0])
    # If slope greater than 45 deg swap x, y
    if abs(math.atan(m)) > (math.pi / 4):
        xyswap = True
        line = [line[1], line[0], line[3], line[2]]
        # Recalculate slope with new order
        if line[2] - line[0] == 0:
            m = (line[3] - line[1]) / .001
        else:
            m = (line[3] - line[1]) / (line[2] - line[0])
    b = line[1] - m * (line[0])
    imgA = np.asarray(img)
    z = imgA[-(iy[-1] + 1):-(iy[0]), ix[0]:ix[-1] + 1]
    if not nc and len(z.shape) == 3:
        # If image, take average of RGB values as point value
        z = np.mean(z, axis=2)
    z = np.flipud(z)
    # numpy arrays are indexed by row, column NOT x, y, but interp object does do x y
    int_pol = RegularGridInterpolator((iy, ix), z, method='linear')

    if line[0] > line[2]:
        xarr = np.arange(int(line[2]), int(line[0]))
    else:
        xarr = np.arange(int(line[0]), int(line[2]))
    yarr = xarr * m + b

    # Grab points from interpolation object
    for i in range(0, xarr.size):
        if not xyswap:
            data.append(int_pol([yarr[i], xarr[i]])[0])
        else:
            data.append(int_pol([xarr[i], yarr[i]])[0])
    if xyswap:
        data = {'x': yarr, 'y': xarr, 'Cut': data}
    else:
        data = {'x': xarr, 'y': yarr, 'Cut': data}
    data = {'x': data['x'].tolist(), 'y': data['y'].tolist(), 'Cut': data['Cut']}

    return data



"""
UI and functionality for NetCDF configuration popup

Creates the NetCDF configuration popup that opens when a user loads a NetCDF file. Ensures that the file
is in a viable configuration for the viewer.
"""

import kivy.uix as ui
from kivy.metrics import dp
from kivy.uix.button import Button
from kivy.uix.popup import Popup
from kivy.uix.label import Label
from kivy.uix.dropdown import DropDown
import nccut.functions as func
import xarray as xr


class NetCDFConfig(Popup):
    """
    UI and functionality for NetCDF configuration popup

    Creates the NetCDF configuration popup that opens when a user loads a NetCDF file. Ensures that the file
    is in a viable configuration for the viewer.

    Attributes:
        home: Reference to root :class:`nccut.homescreen.HomeScreen` instance
        data: xarray.Dataset, Opened NetCDF file
        running (bool): Whether the 'Go' button has been pressed. Used to prevent user from spamming the button.
        var_select: RoundedButton, Variable select button
        var_drop: Dropdown(), Dropdown of variable options
        x_select: Rounded Button, X dimension select button
        y_select: Rounded Button, Y dimension select button
        z_select: Rounded Button, Z dimension select button
        depth_select: Rounded Button, Z dimension value select button
        error: Label for displaying error alerts
        title: Popup title
        content: BoxLayout containing all widgets of the popup
        size_hint (tuple): Tuple (width, height) of relative size of popup to window
    """
    def __init__(self, file, home, **kwargs):
        """
        Defines UI elements and opens popup. By default it selects the first variable of the NetCDF file and if it has
        3 dimensions it selects the first value in the third dimension as the Z value.

        Args:
            file (str): File path of NetCDF file. Must exist and be a valid NetCDF file
            home: Reference to root :class:`nccut.homescreen.HomeScreen` instance
        """
        super(NetCDFConfig, self).__init__(**kwargs)
        self.home = home
        self.data = xr.open_dataset(file)
        self.running = False
        content = ui.boxlayout.BoxLayout(orientation='vertical', spacing=dp(20), padding=dp(20))

        # Variable Selection
        var_box = ui.boxlayout.BoxLayout(spacing=dp(20))
        var_box.add_widget(Label(text="Variable: ", size_hint=(0.3, 1)))
        self.var_select = func.RoundedButton(text=list(self.data.keys())[0], size_hint=(0.7, 1),
                                             halign='center', valign='middle')
        self.var_drop = DropDown()
        for item in list(self.data.keys()):
            btn = Button(text=str(item), size_hint_y=None, height=dp(30), halign='center',
                         valign='middle')
            btn.bind(on_release=lambda btn: self.var_drop.select(btn.text),
                     on_press=self.var_drop.dismiss, size=func.text_wrap)
            self.var_drop.add_widget(btn)
        self.var_drop.bind(on_select=lambda instance, x: self.var_update(x))
        self.var_select.bind(on_release=self.var_drop.open, size=func.text_wrap)
        var_box.add_widget(self.var_select)
        content.add_widget(var_box)

        # X, Y Selection
        dims = list(self.data[self.var_select.text].dims)
        if len(dims) < 3:
            while len(dims) < 3:
                dims.insert(0, "N/A")
        elif len(dims) > 3:
            dims = dims[:3]

        xy_box = ui.boxlayout.BoxLayout(spacing=dp(20))
        xy_box.add_widget(Label(text="X: ", size_hint=(0.2, 1)))
        self.x_select = func.RoundedButton(text=dims[-1], size_hint=(0.3, 1), halign='center', valign='middle')
        self.x_select.bind(on_release=lambda x: self.dim_options(self.x_select), size=func.text_wrap)
        xy_box.add_widget(self.x_select)

        xy_box.add_widget(Label(text="Y: ", size_hint=(0.2, 1)))
        self.y_select = func.RoundedButton(text=dims[-2], size_hint=(0.3, 1), halign='center', valign='middle')
        self.y_select.bind(on_release=lambda x: self.dim_options(self.y_select), size=func.text_wrap)
        xy_box.add_widget(self.y_select)
        content.add_widget(xy_box)

        # Z selection (not always required)
        z_box = ui.boxlayout.BoxLayout(spacing=dp(20))
        z_box.add_widget(Label(text="Z Variable: ", size_hint=(0.2, 1)))
        self.z_select = func.RoundedButton(text=dims[-3], size_hint=(0.3, 1), halign='center', valign='middle')
        self.z_select.bind(on_release=lambda x: self.dim_options(self.z_select),
                           text=self.update_depth_btn, size=func.text_wrap)
        z_box.add_widget(self.z_select)

        z_box.add_widget(Label(text="Z Value: ", size_hint=(0.2, 1)))
        if self.z_select.text == "N/A":
            d_text = "N/A"
        else:
            d_text = str(list(self.data.coords[self.z_select.text].data)[0])
        self.depth_select = func.RoundedButton(text=d_text, size_hint=(0.3, 1), halign='center', valign='middle')
        self.depth_select.bind(size=func.text_wrap, on_release=self.depth_options)
        z_box.add_widget(self.depth_select)
        content.add_widget(z_box)

        # Popup Controls
        c_box = ui.boxlayout.BoxLayout(spacing=dp(20))
        self.error = Label(text="", size_hint=(0.7, 1))
        c_box.add_widget(self.error)
        back = func.RoundedButton(text="Back", size_hint=(0.15, 1))
        back.bind(on_press=self.dismiss)
        c_box.add_widget(back)
        self.go = func.RoundedButton(text="Go", size_hint=(0.15, 1))
        self.go.bind(on_press=self.check_inputs)
        c_box.add_widget(self.go)
        content.add_widget(c_box)

        # Final settings
        self.title = "NetCDF Configuration"
        self.content = content
        self.size_hint = (0.8, 0.8)
        self.bind(on_dismiss=lambda x: self.clean())
        self.open()

    def update_depth_btn(self, *args):
        """
        Updates selected depth option to be the first z value from the currently selected z dimension if variable
        has three dimensions.

        Args:
            *args: Unused args passed by event handler
        """
        if not self.z_select.text == "N/A":
            self.depth_select.text = str(list(self.data.coords[self.z_select.text].data)[0])

    def clean(self):
        """
        Resets file related attributes of the root :class:`nccut.homescreen.HomeScreen` instance
        """
        if not self.home.file_on:
            self.home.clean_file()

    def check_inputs(self, *args):
        """
        Check selected configurations are valid before loading dataset

        When values are selected the button text changes to the selection. This method
        accesses values from the text of the buttons. If all checks are passed sends
        dictionary of configurations to HomeScreen instance and closes popup. Otherwise
        an error message is displayed and popup stays open.
        """
        if not self.running:
            self.running = True
            vals = {'x': self.x_select.text, 'y': self.y_select.text,
                    'z': self.z_select.text, 'z_val': self.depth_select.text,
                    'var': self.var_select.text, 'file': self.data}
            selects = [(self.x_select, "X Dimension"), (self.y_select, "Y Dimension")]
            if len(self.data[self.var_select.text].dims) > 3:
                self.error.text = "This variable has more than 3 dimensions"
                self.running = False
                return
            if len(self.data[self.var_select.text].dims) < 2:
                self.error.text = "This variable has less than 2 dimensions"
                self.running = False
                return
            for sel in selects:
                if sel[0].text == 'N/A':
                    self.error.text = "Please Select a " + sel[1]
                    self.running = False
                    return
            if len(set(list(vals.values())[:-3])) != len(list(vals.values())[:-3]):
                self.error.text = "All X, Y, Z variables must be unique"
                self.running = False
                return
            self.home.load_netcdf(vals)
            self.dismiss()

    def var_update(self, var, *args):
        """
        When a variable is selected updates var_select button text to the variable name. Then finds first
        two or three dimensions of the variable and sets them as the X, Y, and Z (if three) dimension
        selections.

        Args:
            var (str): Variable selected
            *args: Unused arguments passed to method
        """
        setattr(self.var_select, 'text', var)
        dims = list(self.data[self.var_select.text].dims)
        if len(dims) < 3:
            while len(dims) < 3:
                dims.insert(0, "N/A")
        elif len(dims) > 3:
            dims = dims[:3]
        setattr(self.x_select, 'text', dims[-1])
        setattr(self.y_select, 'text', dims[-2])
        setattr(self.z_select, 'text', dims[-3])
        if self.z_select.text == "N/A":
            setattr(self.depth_select, 'text', "N/A")
        else:
            setattr(self.depth_select, 'text', str(list(self.data.coords[self.z_select.text].data)[0]))

    def dim_options(self, dim, *args):
        """
        Creates dropdown for X, Y, or Z dimension selection button. If variable is already selected
        options listed are all dimensions for selected variable. If variable is not selected sets button
        to 'Select ...'

        Args:
            dim: x_select, y_select, or z_select Button
            *args: Unused arguments passed to method
        """
        if not dim.text == "N/A":
            dim_drop = ListDropDown(list(self.data[self.var_select.text].dims), dim)
            dim_drop.open(dim)

    def depth_options(self, *args):
        """
        Creates dropdown for the value of Z dimension if third dimension is selected.

        Args:
            *args: Unused arguments passed to method
        """
        if not self.z_select.text == "N/A":
            depth_drop = ListDropDown(list(self.data.coords[self.z_select.text].data), self.depth_select)
            depth_drop.open(self.depth_select)


class ListDropDown(DropDown):
    """
    Standard dropdown for all selections in the menu.

    Attributes:
        Inherits attributes from kivy.uix.dropdown.Dropdown (see kivy docs)
    """
    def __init__(self, items, button, **kwargs):
        """
        Creates dropdown. When options in dropdown are selected dropdown closes and selection button
        text changes to the selected option.

        Args:
            items: List of items to be options for dropdown.
            button: Button which opens dropdown
        """
        super(ListDropDown, self).__init__(**kwargs)
        for item in items:
            btn = Button(text=str(item), size_hint_y=None, height=dp(30),
                         halign='center', valign='middle', shorten=True)
            btn.bind(on_release=lambda btn: self.select(btn.text), on_press=self.dismiss, size=func.text_wrap)
            self.add_widget(btn)
        self.bind(on_select=lambda instance, x: setattr(button, 'text', x))



"""
This module sets the default logging level for NcCut and defines ways for the user to access and adjust the logging
level. NcCut itself does not implement logging messages but it's dependencies do.
"""


import logging

# Configure the root logger
logging.getLogger().setLevel(logging.WARNING)


def set_logging_level(level):
    """
    Set the logging level for the root logger and specific dependency loggers.
    This allows end users to change the logging level.

    Parameters:
        level (str): The logging level to set (e.g., 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL')
    """
    numeric_level = getattr(logging, level.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError(f'Invalid log level: {level}')

    # Set the level for the root logger
    logging.getLogger().setLevel(numeric_level)


def get_logging_level():
    """
    Get the current logging level for the root logger.

    Returns:
        str: The current logging level (e.g., 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL')
    """
    level = logging.getLogger().level
    return logging.getLevelName(level)
